/**
 * @license Copyright 2016 Google Inc. All Rights Reserved.
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License. You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions and limitations under the License.
 */
'use strict';

const makeComputedArtifact = require('./computed-artifact.js');
const NetworkRecords = require('./network-records.js');

class PushedRequests {
  /**
   * Return list of network requests that were pushed.
   * @param {LH.DevtoolsLog} devtoolsLog
   * @param {LH.Audit.Context} context
   * @return {Promise<Array<LH.Artifacts.NetworkRequest>>}
   */
  static async compute_(devtoolsLog, context) {
    return NetworkRecords.request(devtoolsLog, context).then(records => {
      const pushedRecords = records.filter(r => r.timing && !!r.timing.pushStart);
      return pushedRecords;
    });
  }
}

module.exports = makeComputedArtifact(PushedRequests);
