/**
 * @license Copyright 2017 Google Inc. All Rights Reserved.
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with the License. You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions and limitations under the License.
 */
'use strict';

const Gatherer = require('../gatherer');
const pageFunctions = require('../../../lib/page-functions.js');

class CrawlableLinks extends Gatherer {
  /**
   * @param {LH.Gatherer.PassContext} passContext
   * @return {Promise<LH.Artifacts['CrawlableLinks']>}
   */
  async afterPass(passContext) {
    const expression = `(function() {
      ${pageFunctions.getElementsInDocumentString}; // define function on page
      const resolveURLOrNull = url => {
        try { return new URL(url, window.location.href).href; }
        catch (_) { return null; }
      };

      const selector = 'a[href]:not([rel~="nofollow"])';
      const elements = getElementsInDocument(selector);
      return elements
        .map(node => ({
          href: node.href instanceof SVGAnimatedString ?
            resolveURLOrNull(node.href.baseVal) :
            node.href,
          text: node.href instanceof SVGAnimatedString ?
            node.textContent :
            node.innerText,
        }));
    })()`;

    /** @type {LH.Artifacts['CrawlableLinks']} */
    const links = await passContext.driver.evaluateAsync(expression, {useIsolation: true});
    return links.filter(link => typeof link.href === 'string' && link.href);
  }
}

module.exports = CrawlableLinks;

